/**
 * Helper functions for vortex server and client
 * TLS testing programs.
 */

#include "tls_bug.h"

/** 
 * Function for initializing the vortex library and TLS context.
 */
axl_bool test_vortex_context_init(VortexCtx *ctx,
				const char *private_key,
				const char *public_cert,
				const char *peer_certs)

{
	if (!vortex_init_ctx(ctx)) {
		DEBUG("Vortex library initialization failed...");
		goto err;
	}

	if (!vortex_tls_init(ctx)) {
		DEBUG("Vortex TLS initialization failed...");	
		goto err;
	}

	// tls_config_type located in tls_bug.h
	tls_config *cfg = NULL;
	cfg = calloc(sizeof(tls_config), 1);
	cfg->private_key = strdup(private_key);
	cfg->public_cert = strdup(public_cert);
	cfg->peer_certs = strdup(peer_certs);

	vortex_tls_set_default_ctx_creation(ctx, test_ssl_ctx_creation, cfg);
	vortex_tls_accept_negotiation(ctx, NULL, NULL, NULL);

	return axl_true;
err:
	return axl_false;
}

/**
 * Handler function for setting up the SSL context.
 */
axlPointer test_ssl_ctx_creation(__attribute__((unused))VortexConnection *connection, axlPointer user_data)
{
	SSL_CTX *ssl_ctx;
	tls_config *cfg = (tls_config *) user_data;
	ssl_ctx = SSL_CTX_new(TLSv1_method());

	// Prints expected SSL configuration.
	fprintf(stderr, "\nPeer Certs:  %s \nPublic Cert: %s \nPrivate Key: %s\n\n", cfg->peer_certs, cfg->public_cert, cfg->private_key);
	
	// Checks validity of SSL configuration.
	if (!SSL_CTX_load_verify_locations(ssl_ctx, NULL, cfg->peer_certs)) {
		goto err;
	}
	if (!SSL_CTX_use_certificate_chain_file(ssl_ctx, cfg->public_cert)) {
		goto err;
	}
	if (!SSL_CTX_use_PrivateKey_file(ssl_ctx, cfg->private_key, SSL_FILETYPE_PEM)) {
		goto err;
	}

	// Verifies correctness of SSL configuration.
	SSL_CTX_set_verify(ssl_ctx, SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT, NULL);

	return ssl_ctx;
err:
	DEBUG("Vortex SSL context creation failed...");
	SSL_CTX_free(ssl_ctx);
	return NULL;
}

/**
 * Simple Vortex and TLS initialization.
 * Configurations left to client and server applications.
 */
axl_bool test_simple_init(VortexCtx *ctx)
{
	if (!vortex_init_ctx(ctx)) {
		DEBUG("Context initialization failed...");
		goto err;
	}
	if (!vortex_tls_init(ctx)) {
		DEBUG("TLS initialization failed...");
		goto err;
	}

	return axl_true;
err:
	return axl_false;
}

/**
 * Status handler for TLS connection.
 * Used for simple configuration.
 */
void tls_status_handler(VortexConnection *v_conn,
			VortexStatus v_status,
			const char *status_message,
			axlPointer user_data)
{
	switch (v_status) {
	case VortexOk:
		DEBUG("TLS negotiation is valid over this connection.");
		fprintf(stderr, "%s\n", status_message); 
		break;
	case VortexError:
		DEBUG("TLS negotiation has failed.");
		fprintf(stderr, "%s\n", status_message); 
		break;
	}
	return;
}
